/*
*  GUI <--->  Core protocol
*
*/


#ifndef ed2k_gui_gui_core_protocol_h_included
#define ed2k_gui_gui_core_protocol_h_included

/* messages from the core to the GUI
*/

#define CORE_SERVER_LIST		170	/* the server list follows 		*/
//	filelist: server list

#define CORE_FRIEND_LIST		171	/* the friend list follows 		*/
//	filelist: friend list

#define CORE_SHARED_DIRS		172 	/* the list of shared follows 		*/
//	w16: num directories
//	string: directory name

#define CORE_SHARED_FILES		173
//	filelist: shared files

#define CORE_DOWNLOAD_GAP_DETAILS	174	// Jaroslav/Tim: the download gap details follow
// 16 bytes: hash
// w16: number of gaps
// for each gap:
//   w32 gap start
//   w32 gap end
//   w16 gap status

#define CORE_CLIENT_STATS		175	// Tim
// float: free space on temp drive in MB
// float: free space in incoming in MB
// float: space needed by downloads in MB
// w32:   client ID
// w16:   no. of connections used currently
// w16:   no. of people on queue

#define CORE_STATUS_MSG			180
/*	string: message
*/

#define CORE_ERROR_MSG			181
/*	string: error message
*/

#define CORE_CONNECTED_TO		182
/*	string: server name
*/

#define CORE_DISCONNECTED		183


#define CORE_SET_SERVER_STATS		184
/*	w32: files
	w32: users
*/

#define CORE_EXTENDING_SEARCH		185
/*	string: server name
*/

#define CORE_SEARCH_RESULT		186
/*	meta: result meta
*/

#define CORE_NEW_SEARCH_RESULTS		187
/*	filelist: result list
	w8: more or not (0 or 1)
*/

#define CORE_NEW_DOWNLOAD		188
/*	meta: download file
	w8: initial preference (???)
	i4string: temp file name
*/

#define CORE_REMOVE_DOWNLOAD		189
/*	hash: fileID
*/

#define CORE_NEW_UPLOAD			190
/*	string: file name
	meta: upload user
*/

#define CORE_REMOVE_UPLOAD		191
/*	hash: uploader/user ID
*/

#define CORE_NEW_UPLOAD_SLOT		192
/*	w32: slot ID
	string: uploader name
*/

#define CORE_REMOVE_UPLOAD_SLOT		193
/*	w32: slot ID
*/

#define	CORE_USER_FILES			194
/*	filelist: users filelist
*/

#define CORE_HASHING			195
/*	string: filename
*/

#define CORE_FRIEND_LIST_UPDATE		196	/* the friend list needs to be updated 	*/


#define CORE_DOWNLOAD_STATUS		197
/*	w16: number of downloads
	...
	w16: download ID
	w8: status
	float: current speed (kB/sec)
	w32: ammount transfered
	w8: % available
	w8: num sources
	...

*/

#define CORE_UPLOAD_STATUS		198
/*	w16: number of uploads
	...
	w16: uploadID
	float: current speed (kB/sec)
	...

*/

#define CORE_OPTIONS			199	/* options follow */

/*
w16	clientversion
float	max down
float	max up
w16	doorport
w16	max connections
string	client name
string	temp dir
string	incoming dir
w8	autoConnect
w8	autoServRemove
w8	primsgallow
w8	savecorrupt
w8	verifyCancel
w16     adminDoorPort
w32	core build date
float	line down speed		// Tim
*/


/********************************************************************************************
*    messages from the GUI to the core
*********************************************************************************************/

#define CONT_LOGIN			100
/*      string: username
        string: password
*/

#define CONT_STOP			101	/* core please stop 			*/

#define CONT_CMD			102
/*	string: commandline command
*/


#define GUI_CONNECT				200
/*	w32: server ID 0 for any server
	w16: server port
*/


#define GUI_DISCONNECT			201	/* cl */

#define GUI_SEARCH			202
/*	search: the search
*/


#define GUI_EXSEARCH			203 	/* cl */

#define GUI_MORESEARCH			204 	/* cl */

#define GUI_SEARCH_USER			205
/*	search: the search
*/


#define GUI_EXSEARCH_USER		206

#define GUI_DOWNLOAD			207
/*	hash: fileID
*/


#define GUI_PAUSE_DOWNLOAD		208
/*	hash: fileID
*/


#define GUI_RESUME_DOWNLOAD		209
/*	hash: fileID
*/


#define GUI_CANCEL_DOWNLOAD		210
/*	hash: fileID
*/


#define GUI_SETPRI_DOWNLOAD		211
/*	hash: fileID
	w8: priority (0 low, 1 normal, 2 high)
*/


#define GUI_VIEW_FRIEND_FILES	212

#define GUI_GET_SERVERLIST		213

#define GUI_GET_FRIENDLIST		214

#define GUI_GET_SHARE_DIRS		215

#define GUI_SET_SHARE_DIRS		216

#define GUI_START_DL_STATUS		217		/* the core will send you the status of the downloads periodically */

#define GUI_STOP_DL_STATUS		218		/* the core will stop sending you the status of the downloads */

#define GUI_START_UL_STATUS		219

#define GUI_STOP_UL_STATUS		220

#define GUI_DELETE_SERVER		221
/*	w32: serverID
	w16: server port
*/


#define GUI_ADD_SERVER			222		/* cl */
/*	w32: serverID
	w16: server port
*/

#define GUI_SETPRI_SERVER		223
/*	w32: serverID
	w16: server port
	w8: preference (0-NORMAL, 1-HIGH, 2-LOW)
*/

#define GUI_GET_SHARE_FILES		224

#define GUI_GET_OPTIONS			225

#define GUI_NEW_DOWNLOAD		226		// for ed2k-links - new, new, 'cause not in searchlist
// hash
// 4+2 bytes unused
// taglist (i.e. at least filename+filesize)

#define GUI_GET_DOWNLOAD_GAP_DETAILS	227		// Jaroslav/Tim
// hash
// w16: number of gaps
// - each gap:
// 	w32 start
// 	w32 end
// 	w16 value

#define GUI_GET_CLIENT_STATS		228		// Tim
// float - free space in tmp in MB
// float - free space in incoming in MB
// float - space needed in MB
// w32 - clientID
// w16 - current connections
// w16 - currently on queue


#define CORE_FILECHUNK			253		// for testing purposes
// filehash
// w32 from
// w32 to
// data

#define CORE_FILECOMPLETE		254		// for testing purposes
// filehash
// i4string: filename

/*


hash:
	16 bytes

string:
	w16: length of the string
	var: the string

metaTagInt:
metaTagString:
metaTagHash:

meta:

filelist:

*/

#endif
