/* mainwindow.c
 *
 * part of ed2k-gtk-gui
 *
 * (c) 2001-2003 Tim-Philipp Muller <t.i.m@orange.net>
 *
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "global.h"
#include "connect_gui.h"
#include "colors.h"
#include "gui_options.h"
#include "icons.h"
#include "mainwindow.h"
#include "misc.h"
#include "misc_gtk.h"
#include "notebook.h"
#include "search_page.h"
#include "shared_page.h"
#include "statusbar.h"
#include "status_page.h"
#include "toolbar.h"
#include "uploads_page.h"
#include "gui_flags.h"

#include "pipe.h"

#include <gtk/gtk.h>
#include <ctype.h>

/* global variables */

GtkWidget *window = NULL;	/* main window widget */


/* local variables */

enum
{
 TARGET_STRING,
 TARGET_ROOTWIN,
 TARGET_URL
};

static GtkTargetEntry dragtypes[] =
{
  { N_("STRING"),     0, TARGET_STRING },
  { N_("text/plain"), 0, TARGET_STRING },
  { N_("text/uri-list"), 0, TARGET_URL },
  { N_("application/x-rootwin-drop"), 0, TARGET_ROOTWIN }
};


/* mainwindow destroy
 *
 *
 */

void
mainwindow_retrieve_some_options_before_main_window_is_destroyed(void)
{
/*	deprecated:
	paned_win_position [UL_PANE_POS] = (GTK_HPANED(ulpanedwin)->paned).child1_size; */

	paned_win_position [SHARED_PANE_POS] = (GTK_HPANED(spanedwin)->paned).child1_size;
	paned_win_position [UL_PANE2_POS] = (GTK_VPANED(ulpanedwin2)->paned).child1_size;
	if (window) {
		gdk_window_get_position (window->window, (gint *) &main_window_size.x, (gint *) &main_window_size.y);
		/* we read gdk win pos, but later we set gtk win pos... */
		main_window_size.x -= X_POS_CORRECT;
		main_window_size.y -= Y_POS_CORRECT;
		gdk_window_get_size     (window->window, (gint *) &main_window_size.width, (gint *) &main_window_size.height);
	}
}

/* Note: widget and data can be NULL (if we want to quit w/o asking) */
void
mainwindow_delete_event_really_quit (GtkWidget *widget, gpointer data)
{
	connection_closed_deliberately = TRUE;
	about_to_quit = TRUE;
	clean_up_stuff();
	mainwindow_retrieve_some_options_before_main_window_is_destroyed();
	if (data) gtk_widget_destroy (GTK_WIDGET (data));	// destroy the yes/no dialog

	main_clean_up_connections_and_callbacks_before_quit ();

	if (window)
	{
//		gtk_widget_destroy(window);
//		window = NULL;
	}

	gtk_main_quit ();
}

void
mainwindow_delete_event_donot_quit (GtkWidget *widget, gpointer data)
{
        gtk_widget_destroy (GTK_WIDGET (data));	// destroy the yes/no dialog
}


void
mainwindow_delete_event (GtkWidget *widget, gpointer data)
{
	gchar msg[512];

	if (option_verify_exit == 1)
	{
		g_snprintf (msg, 512, _("Do you really want to quit?\n\nCore will be %s\n"),
			  (option_shutdown_core_on_exit==1) ? _("shutdown.") : _("kept running."));
                yesno_dialog (_(" CLOSE APPLICATION "), msg, _(" yes "), _(" no "),
			       mainwindow_delete_event_really_quit, mainwindow_delete_event_donot_quit);
	} else mainwindow_delete_event_really_quit (NULL, NULL);
}

//
// Drag'n'Drop stuff
//

/*** the next two routines are taken straight from gnome-libs so that the
     gtk-only version can receive drag and drops as well ***/

/**
 * gnome_uri_list_extract_uris:
 * @uri_list: an uri-list in the standard format.
 *
 * Returns a GList containing strings allocated with g_malloc
 * that have been splitted from @uri-list.
 */
static GList*
gnome_uri_list_extract_uris (const gchar* uri_list)
{
	const gchar *p, *q;
	gchar *retval;
	GList *result = NULL;

	g_return_val_if_fail(uri_list != NULL, NULL);

	p = uri_list;

	/* We don't actually try to validate the URI according to RFC
	 * 2396, or even check for allowed characters - we just ignore
	 * comments and trim whitespace off the ends.  We also
	 * allow LF delimination as well as the specified CRLF.
	 */
	while (p) {
		if (*p != '#') {
			while (isspace((int)(*p)))
				p++;

			q = p;
			while (*q && (*q != '\n') && (*q != '\r'))
				q++;

			if (q > p) {
			        q--;
				while (q > p && isspace((int)(*q)))
					q--;

				retval = (char *)g_malloc(q - p + 2);
				strncpy (retval, p, q - p + 1);
				retval[q - p + 1] = '\0';

				result = g_list_prepend(result, retval);
			}
		}
		p = strchr(p, '\n');
		if (p)
			p++;
	}

	return g_list_reverse(result);
}


/**
 * gnome_uri_list_extract_filenames:
 * @uri_list: an uri-list in the standard format
 *
 * Returns a GList containing strings allocated with g_malloc
 * that contain the filenames in the uri-list.
 *
 * Note that unlike gnome_uri_list_extract_uris() function, this
 * will discard any non-file uri from the result value.
 */
static GList*
gnome_uri_list_extract_filenames (const gchar* uri_list)
{
	GList *tmp_list, *node, *result;

	g_return_val_if_fail (uri_list != NULL, NULL);

	result = gnome_uri_list_extract_uris (uri_list);

	tmp_list = result;
	while (tmp_list) {
		gchar *s = (char *)tmp_list->data;

		node = tmp_list;
		tmp_list = tmp_list->next;

//		if (!strncmp (s, "file:", 5)) {
			node->data = g_strdup (s);
//		} else {
//			result = g_list_remove_link(result, node);
//			g_list_free_1 (node);
//		}
		g_free (s);
	}
	return result;
}

// this comes from gnotepad+
//
void win_drag_data_recv(GtkWidget *wgt, GdkDragContext *context, int x, int y,
		   GtkSelectionData *seldata, guint info, guint time,
		   gpointer cbdata)
{
	GList *fnames, *fnp;
	guint count;
	char *buf = NULL;
	unsigned int bufsize = 0, len;

	fnames = gnome_uri_list_extract_filenames((char *)seldata->data);
	count = g_list_length(fnames);
	if (count > 0) {
		status_msg (_("GUI: received drag'n'drop.\n"));

		for (fnp = fnames; fnp; fnp = fnp->next, count--) {
			len = strlen((char *)(fnp->data));
			buf = realloc (buf, bufsize+len+1);
			strcpy (buf+bufsize,(char *)(fnp->data));
			bufsize += len;
		}
		buf[bufsize]=0x00;
		pipe_parse_message (buf);
		if (buf) free (buf);
	}
	// Releases all of the resources allocated by @list.
	g_list_foreach (fnames, (GFunc)g_free, NULL);
	g_list_free (fnames);

} /* win_drag_data_recv */


static void
mainwindow_size_allocate_signal (GtkWidget *widget, gpointer data)
{
	if ((window) && (window->window))
	{
		gint width, height;
		gdk_window_get_size (window->window, &width, &height);
		if (width<720)
		{
			gtk_widget_hide (button_forget);
		} else gtk_widget_show (button_forget);
	}

}


/******************************************************************************
 *
 *   mainwindow_set_title
 *
 */

void
mainwindow_set_title (const gchar *networkname)
{
	gchar *wintitle;

	g_return_if_fail (networkname!=NULL);

	wintitle = g_strdup_printf (N_("%s (%u)"), networkname, option_gui_instance);
	g_return_if_fail (wintitle!=NULL);

	if (option_gui_instance<=0)
	{
		gchar *space = strchr(wintitle, ' ');
		if (space)
		{
			*space = 0x00;
		}
	}

	gtk_window_set_title (GTK_WINDOW (window), wintitle);

	g_free(wintitle);

}


/******************************************************************************
 *
 *   mainwindow_create
 *
 *   Creates the main window which is going to contain the notebook and the logo
 *
 */

void
mainwindow_create (void)
{
	GtkWidget	*vbox;

	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	g_assert (window!=NULL);

	gtk_widget_show (window);
	gtk_container_border_width (GTK_CONTAINER(window), 2);
	gtk_widget_set_usize (window, 640, 400);

	mainwindow_set_title (N_("eDonkey2000"));

	/* destroy, delete and resize events */
	gtk_signal_connect (GTK_OBJECT (window), N_("delete_event"),
				GTK_SIGNAL_FUNC (mainwindow_delete_event), NULL);
	gtk_signal_connect (GTK_OBJECT (window), N_("size-allocate"),
				GTK_SIGNAL_FUNC (mainwindow_size_allocate_signal), NULL);
	gtk_signal_connect (GTK_OBJECT (window), N_("destroy"),
				GTK_SIGNAL_FUNC(gtk_widget_destroyed), &window);

	/* set up drag'n'drop */
	gtk_drag_dest_set(window, GTK_DEST_DEFAULT_ALL, dragtypes, 3,
				GDK_ACTION_COPY | GDK_ACTION_MOVE);
	gtk_signal_connect(GTK_OBJECT(window), N_("drag_data_received"),
				GTK_SIGNAL_FUNC(win_drag_data_recv), NULL);

	/* set position and size as read in from options file */
	gdk_window_move_resize (window->window, main_window_size.x, main_window_size.y,
				main_window_size.width, main_window_size.height);

	/* create vbox to hold toolbar, notebook and statusbar */
	vbox = gtk_vbox_new (FALSE, 0);
	g_assert (vbox!=NULL);
	gtk_widget_show (vbox);
	gtk_container_add (GTK_CONTAINER (window), vbox);
	gtk_box_pack_start (GTK_BOX (vbox), toolbar_create(), FALSE, FALSE, 0);

	/* create the window's notebook */
	notebook = notebook_create (window, option_main_notebook_tabs_pos);
	g_assert (notebook!=NULL);
	gtk_widget_show(notebook);
	gtk_box_pack_start (GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), statusbar_create(), FALSE, FALSE, 0);

	if (option_nogreen == 0)
	{
		gtk_widget_set_style (window, style_dark_green_background);
		statusbar_set_styles();
		toolbar_set_styles();
	}

	if (option_no_app_icon==0)	/* this option can only be unset via the command line */
	{
		if (icon_app)
		{
			GdkPixmap	*pixmap = NULL;
			GdkBitmap	*mask = NULL;

			pixmap = gdk_pixmap_create_from_xpm_d (
						window->window,
						&mask,
    					&window->style->bg[GTK_STATE_NORMAL],
						icon_app);

			gdk_window_set_icon (window->window, NULL, pixmap, mask);
		}
	}

}
