/*
*	options_page.c
*
*	creates the options page in the notebook and deals with the controls...
*
*	(c) 2001 Tim-Philipp Muller <t.i.m@orange.net>
*
*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <time.h>

#include "downloads_data.h"
#include "global.h"
#include "gui_flags.h"
#include "gui_options.h"
#include "icons.h"
#include "linkhandler.h"
#include "mainwindow.h"
#include "misc.h"
#include "misc_gtk.h"
#include "notebook.h"
#include "options_page.h"
#include "status_page.h"
#include "http_get.h"
#include "send_command.h"
#include "statusbar.h"

#define GNET_EXPERIMENTAL
#include <gnet/gnet.h>


enum
{
 OPTIONS_LINKHANDLER_INSTALL_KDE_HANDLER = 1,
 OPTIONS_LINKHANDLER_INSTALL_GNOME1_HANDLER,
 OPTIONS_LINKHANDLER_INSTALL_GNOME2_HANDLER,
} ;

/* global variables */

/* checkbox widgets */
static GtkWidget	*cb_verbose;
static GtkWidget	*cb_verifyexit, *cb_shutdowncoreonexit;
static GtkWidget	*cb_connectcore, *cb_spawncore;
static GtkWidget	*cb_restartcrashedcore;
static GtkWidget	*cb_notabtext;
static GtkWidget	*cb_linkdownload, *cb_nodoubleclick;
static GtkWidget	*cb_nogreen, *cb_slowcon, *cb_logtimestamp;
static GtkWidget	*cb_nolinkstolog;

/* entry widgets */
static GtkWidget	*ew_incoming, *ew_temporary, *ew_nickname;
static GtkWidget	*ew_defaulthost, *ew_defaultuser, *ew_defaultpass;
static GtkWidget	*ew_defaultdonkeypath;
static GtkWidget	*ew_slistURL, *ew_slistURL2, *ew_completecommand;

static GtkWidget	*spinbutton_minfreespace = NULL;
static GtkWidget	*spinbutton_serverrotate = NULL;
static GtkWidget	*spinbutton_clearswin = NULL;
static GtkWidget	*spinbutton_defaultport = NULL;

static GtkWidget	*spinbutton_maxdlspeed = NULL;
static GtkWidget	*spinbutton_maxulspeed = NULL;
static GtkWidget	*spinbutton_port = NULL;
static GtkWidget	*spinbutton_adminport = NULL;
static GtkWidget	*spinbutton_maxcon = NULL;

/* radio buttons */
static GtkWidget	*radio1_lh = NULL;
static GtkWidget	*radio2_lh = NULL;
static GtkWidget	*radio3_lh = NULL;
	

// functions

static GtkWidget	*options_page_make_linkhandler_options_frame (GtkTooltips *tooltips);



/*******************************************************************************
 *
 *   options_update_option_widgets
 *
 */
 
void 
options_update_option_widgets (void)
{
	gtk_entry_set_text (GTK_ENTRY(ew_nickname),   option_nickname);
	gtk_entry_set_text (GTK_ENTRY(ew_incoming),   option_incoming);
	gtk_entry_set_text (GTK_ENTRY(ew_temporary),  option_temporary);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(spinbutton_maxdlspeed), option_max_dl_speed);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(spinbutton_maxulspeed), option_max_ul_speed);

	gtk_spin_button_set_value (GTK_SPIN_BUTTON(spinbutton_port),       option_doorport * 1.0);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(spinbutton_adminport),  option_adminport * 1.0);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(spinbutton_maxcon),     option_max_connections * 1.0);

	gtk_entry_set_text (GTK_ENTRY(ew_defaulthost), (option_defaulthost) ? option_defaulthost : "");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(spinbutton_defaultport),option_defaultport * 1.0);
	gtk_entry_set_text (GTK_ENTRY(ew_defaultuser), (option_defaultuser) ? option_defaultuser : "");
	gtk_entry_set_text (GTK_ENTRY(ew_defaultpass), (option_defaultpass) ? option_defaultpass : "");
	gtk_entry_set_text (GTK_ENTRY(ew_defaultdonkeypath), (option_defaultdonkeypath) ? option_defaultdonkeypath : "");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(spinbutton_serverrotate), option_server_rotate*1.0);
	gtk_entry_set_text (GTK_ENTRY(ew_slistURL), (option_serverlist_URL) ? option_serverlist_URL : "");
	gtk_entry_set_text (GTK_ENTRY(ew_slistURL2), (option_serverlist_URL2) ? option_serverlist_URL2 : "");
	gtk_entry_set_text (GTK_ENTRY(ew_completecommand), (option_exec_on_complete) ? option_exec_on_complete : "");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(spinbutton_clearswin), option_clearstatuswindow*1.0);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(spinbutton_minfreespace), option_minfreetempspace*1.0);

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_verbose), (option_verbose==1));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_verifyexit), (option_verify_exit==1));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_shutdowncoreonexit), (option_shutdown_core_on_exit==1));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_spawncore), (option_spawn_core_on_startup==1));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_connectcore), (option_connect_to_core_on_startup==1));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_notabtext), (option_main_notebook_tabs_hide_text==1));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_linkdownload), (option_download_ed2k_link_immediately==1));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_restartcrashedcore), (option_restart_crashed_core==1));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_nodoubleclick), (option_disable_doubleclick==1));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_nogreen), (option_nogreen==1));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_slowcon), (option_slowconnection==1));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_logtimestamp), (option_logtimestamp==1));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_nolinkstolog), (option_nolinkstolog==1));

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio1_lh), (option_linkhandler_what_to_do_if_no_GUI_running == LINKHANDLER_IF_NO_GUI_ASK));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio2_lh), (option_linkhandler_what_to_do_if_no_GUI_running == LINKHANDLER_IF_NO_GUI_START_GUI));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio3_lh), (option_linkhandler_what_to_do_if_no_GUI_running == LINKHANDLER_IF_NO_GUI_REMEMBER_AND_QUIT));
}


void
options_ok_button_clicked (GtkWidget *widget, gpointer gdata)
{
	// the core options flags we have to copy in dummy variable, because we have to toggle the flags
        gboolean verbose		= gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_verbose          ));

        gchar    *incoming         = gtk_editable_get_chars (GTK_EDITABLE(ew_incoming), 0, -1);
        gchar    *temporary        = gtk_editable_get_chars (GTK_EDITABLE(ew_temporary), 0, -1);
	gchar	 *nickname	   = gtk_editable_get_chars (GTK_EDITABLE(ew_nickname), 0, -1);

	gchar    *defaulthost      = gtk_editable_get_chars (GTK_EDITABLE(ew_defaulthost), 0, -1);
	gchar    *defaultuser      = gtk_editable_get_chars (GTK_EDITABLE(ew_defaultuser), 0, -1);
	gchar    *defaultpass      = gtk_editable_get_chars (GTK_EDITABLE(ew_defaultpass), 0, -1);
	gchar    *defaultpath      = gtk_editable_get_chars (GTK_EDITABLE(ew_defaultdonkeypath), 0, -1);
	gchar	 *slistURL	   = gtk_editable_get_chars (GTK_EDITABLE(ew_slistURL), 0, -1);
	gchar	 *slistURL2	   = gtk_editable_get_chars (GTK_EDITABLE(ew_slistURL2), 0, -1);
	gchar	 *completecommand  = gtk_editable_get_chars (GTK_EDITABLE(ew_completecommand), 0, -1);

//	char     *dummy;                /* to store string conversion error state from strtol */
//	gint     i_defport;
	gint	 dummyint;
	GString  *error_msg;             /* to store our error messages                        */

	last_time_of_user_click = time(NULL);

	// the GUI options flags we can read directly
	option_verify_exit 				= gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_verifyexit       ));
	option_shutdown_core_on_exit	= gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_shutdowncoreonexit));
	option_spawn_core_on_startup	= gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_spawncore));
	option_connect_to_core_on_startup	= gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_connectcore));
	option_download_ed2k_link_immediately	= gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_linkdownload));
	option_restart_crashed_core			= gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_restartcrashedcore));
	option_disable_doubleclick			= gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_nodoubleclick));
	option_nogreen						= gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_nogreen));
	option_slowconnection				= gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_slowcon));
	option_logtimestamp					= gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_logtimestamp));
	option_nolinkstolog					= gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_nolinkstolog));
	option_main_notebook_tabs_hide_text	= gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_notabtext));
	// checking validity of options (initial string for error messages)

	error_msg = g_string_new (N_(""));

	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio1_lh)))
	{
		option_linkhandler_what_to_do_if_no_GUI_running = LINKHANDLER_IF_NO_GUI_ASK;
	}
	else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio2_lh)))
	{
		option_linkhandler_what_to_do_if_no_GUI_running = LINKHANDLER_IF_NO_GUI_START_GUI;
	}
	else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio3_lh)))
	{
		option_linkhandler_what_to_do_if_no_GUI_running = LINKHANDLER_IF_NO_GUI_REMEMBER_AND_QUIT;
	}
	else
	{
		error_msg = g_string_append (error_msg, N_("linkhandler radio button inconsistency"));
	}
		
	/* check if nickname is empty string */
	if (nickname[0]==0x00) {
		g_string_append (error_msg, _("You have to enter a nickname.\n"));
	} else {
		if (strcmp(nickname, option_nickname)!=0) {
			send_set_nickname (nickname);
			ok_dialog (_("options - change nickname"), _("Your new nickname will take effect next time you connect to a server."));
		}
	}

	send_set_max_speeds (gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_maxdlspeed)),
							gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_maxulspeed)));

	/* check if incoming directory exists and we have read and write permission */

	if (strcmp (incoming, option_incoming) != 0 )
	{
		if ((strcmp (clienthost, N_("localhost"))==0) && (access(incoming, F_OK) != 0))
		{
			g_string_append (error_msg, _("incoming directory does not exist.\n"));
		} else {
			if ((strcmp (clienthost, N_("localhost"))==0) && (access(incoming, R_OK | W_OK) != 0)) {
				g_string_append (error_msg,
					_("incoming directory - we don't have both read and write permission.\n"
					   "but I'll change it anyway\n"
					   "(in case the core runs under a different user ID than the GUI)\n\n"));
			}
			if (strcmp (incoming, option_incoming) != 0)
				send_set_incoming (incoming);
		}
	}

	/* check if temporary directory exists and we have read and write permission */

	if (strcmp (temporary, option_temporary) != 0 )
	{
		if ((strcmp (clienthost, N_("localhost"))==0) && (access(temporary, F_OK) != 0))
		{
			g_string_append (error_msg, _("temporary directory does not exist.\n"));
		} else {
			if ((strcmp (clienthost, N_("localhost"))==0) && (access(temporary, R_OK | W_OK) != 0))
			{
				g_string_append (error_msg,
						_("temporary directory - we don't have both read and write permission.\n"
						   "but I'll change it anyway\n"
						   "(in case the core runs under a different user ID than the GUI)\n\n"));
			}
			if (downloads_get_num()>0)
			{
				g_string_append (error_msg, _("Can't change temp folder while there are unfinished downloads.\n"));
			} else send_set_temp (temporary);
		}
	}

	dummyint = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spinbutton_port));
	if (option_doorport != dummyint)
	{
		send_set_port (dummyint);
		status_msg (_("GUI: listening port changed - will take effect on restart (of the core).\n"));
		if (dummyint<1024)
		{
			g_string_append (error_msg, _("\nWarning: You have chosen a port <1024 to listen to.\n"
											"This usually requires root privileges (just thought I'd point that out).\n\n"));
		}
	}

	dummyint = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spinbutton_adminport));
	if (option_adminport != dummyint)
	{
		send_set_admin_port (dummyint);
		status_msg (_("GUI: admin listening port changed - will take effect on restart (of the core).\n"));
		if (dummyint<1024)
		{
			g_string_append (error_msg, _("\nWarning: You have chosen an admin port <1024 to listen to.\n"
											"This usually requires root privileges (just thought I'd point that out).\n\n"));
		}
	}

	dummyint = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spinbutton_maxcon));
	if (option_max_connections != dummyint)
		send_set_max_connections (dummyint);

	// default host to connect GUI to
	if ( (!option_defaulthost) || ((option_defaulthost) && strcmp(defaulthost, option_defaulthost)!=0))
	{
		if (defaulthost[0]==0x00)
		{
			g_string_append (error_msg, _("You have to enter a default host to connect the controller to (use 'localhost' for a local core).\n"));
		} else {
			strrealloccpy (&option_defaulthost, defaulthost);
		}
	}

	// default port to connect GUI controller to core
	option_defaultport = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spinbutton_defaultport));

	// default username and password (simply copy)
	strrealloccpy (&option_defaultuser, defaultuser);
	strrealloccpy (&option_defaultpass, defaultpass);

	// path to donkey binary (ignore checking if it's empty - might be on purpose)
		if ((defaultpath) && (defaultpath[0]!=0x00))
		{
			if (access(defaultpath, F_OK) == 0)
			{
				if (access(defaultpath, R_OK | X_OK) != 0)
				{
					g_string_append (error_msg, "You don't have read/exec permission for the donkey binary.\n");
				} else strrealloccpy (&option_defaultdonkeypath, defaultpath);
			} else g_string_append (error_msg, "The default path to the donkey binary does not exist on this computer.\n");
        } else strrealloccpy (&option_defaultdonkeypath,"");

	option_server_rotate = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spinbutton_serverrotate));
	if (option_server_rotate>0 && option_server_rotate<30)
		option_server_rotate=30;	// make sure it's not too extreme

	option_minfreetempspace = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spinbutton_minfreespace));

	if ( (!option_serverlist_URL) || ((option_serverlist_URL) && strcmp (slistURL, option_serverlist_URL) != 0))
	{
		// check if URL is valid
		GURL	*checkurl = gnet_url_new (slistURL);

		if (misc_gnet_http_url_okay(checkurl))
		{
			char *urlcopy = NULL;
			strrealloccpy (&urlcopy, slistURL);
			check_for_unwanted_URL ( & urlcopy );
			strrealloccpy (&option_serverlist_URL, urlcopy);
			if (urlcopy) free (urlcopy);
		} else g_string_append (error_msg, _("There was an error parsing the serverlist URL (look at status window for details).\n"));
		if (checkurl) gnet_url_delete(checkurl);
	}

	if ((!option_serverlist_URL2) || ((option_serverlist_URL2) && strcmp (slistURL2, option_serverlist_URL2) != 0))
	{
		// check if URL is valid
		GURL	*checkurl = gnet_url_new (slistURL2);
		if (misc_gnet_http_url_okay(checkurl))
		{
			char *urlcopy = NULL;
			strrealloccpy (&urlcopy, slistURL2);
			check_for_unwanted_URL ( & urlcopy );
			strrealloccpy (&option_serverlist_URL2, urlcopy);
			if (urlcopy) free (urlcopy);
		} else g_string_append (error_msg, _("There was an error parsing the second serverlist URL (see status window).\n"));
		if (checkurl) gnet_url_delete(checkurl);
	}

	strrealloccpy (&option_exec_on_complete, completecommand);

	option_clearstatuswindow = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spinbutton_clearswin));

	// now toggle flag options if necessary
	if (verbose != option_verbose)
	{
		option_verbose = (~option_verbose) & 1;
		send_verbose (option_verbose);
	}

	notebook_show_hide_tab_label_text ();

	// if there were errors, pop up dialog window

	if (error_msg->len != 0)
	{
		ok_dialog (_("options page input error"), error_msg->str);
	}
	else
	{
		mainwindow_retrieve_some_options_before_main_window_is_destroyed();
		write_gui_options();
	}
	
	
	g_string_free (error_msg, TRUE);

	send_get_options();	// which in turn will update the widgets..

	if (incoming) g_free(incoming);
	if (temporary) g_free(temporary);
	if (nickname) g_free(nickname);
	if (defaulthost) g_free(defaulthost);
	if (defaultuser) g_free(defaultuser);
	if (defaultpass) g_free(defaultpass);
	if (defaultpath) g_free(defaultpath);
	if (slistURL) g_free(slistURL);
	if (slistURL2) g_free(slistURL2);
	if (completecommand) g_free(completecommand);
}


static void
options_cancel_button_clicked (GtkWidget *widget, gpointer gdata)
{
	options_update_option_widgets ();
}

static void
options_refresh_button_clicked (GtkWidget *widget, gpointer gdata)
{
	send_get_options();
}

static void
options_rotate_tabs_button_clicked (GtkWidget *widget, gpointer gdata)
{
	switch (option_main_notebook_tabs_pos)
	{
		case GTK_POS_LEFT:		option_main_notebook_tabs_pos = GTK_POS_TOP;
		break;
		case GTK_POS_RIGHT:		option_main_notebook_tabs_pos = GTK_POS_BOTTOM;
		break;
		case GTK_POS_TOP:		option_main_notebook_tabs_pos = GTK_POS_RIGHT;
		break;
		case GTK_POS_BOTTOM:	option_main_notebook_tabs_pos = GTK_POS_LEFT;
		break;
	}

	if (notebook)
		gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), (GtkPositionType)option_main_notebook_tabs_pos);

	notebook_show_hide_tab_label_text ();

}

/* options_icontheme_selected
 *
 *
 *
 *
 */

static void
options_icontheme_selected (GtkWidget *widget, gpointer data)
{
	g_return_if_fail (data!=NULL);

	/* XXX - free! */
	if (option_iconthemedir)
		free(option_iconthemedir);	/* XXX - was read in with read_string_from_disk() */

	/* XXX - strdup! */
	option_iconthemedir = strdup((gchar*)data);	/* XXX - see above; so we can use free() */

	statusbar_extra_message (_(" The new icon theme ('%s') will be active after the next GUI restart."), g_basename((gchar*)data));
} ;


/* options_page_make_lookfeel_frame
 *
 * creates the 'GUI Look and Feel' frame and returns it
 *
 */

static GtkWidget *
options_page_make_lookfeel_frame (GtkTooltips *tooltips)
{
	GtkWidget *guiopt_look_frame;
	GtkWidget *guiopt_look_theme_frame;
	GtkWidget *om_iconthemes;
	GtkWidget *vbox9;
	GtkWidget *guiopt_look_table;
	GtkWidget *vbox10;
	GtkWidget *hbox3;
	GtkWidget *rotate_button;
	GtkWidget *hbox5;
	GtkWidget *rotate_buttonbox;
	GSList    *omdirs;

	guiopt_look_frame = gtk_frame_new (_(" GUI Look and Feel Options "));
	gtk_widget_show (guiopt_look_frame);
//	gtk_container_set_border_width (GTK_CONTAINER(guiopt_look_frame),5);

	vbox9 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox9);
	gtk_container_add (GTK_CONTAINER (guiopt_look_frame), vbox9);

	guiopt_look_table = gtk_table_new (2, 2, FALSE);
	gtk_widget_show (guiopt_look_table);
	gtk_box_pack_start (GTK_BOX (vbox9), guiopt_look_table, TRUE, TRUE, 0);

	cb_notabtext = gtk_check_button_new_with_label (_("hide text in non-horiz tabs"));
	SET_TOOLTIP(cb_notabtext, _("Only icons will appear in tabs when they are not set to horizontal layout"));
	gtk_widget_show (cb_notabtext);
	gtk_table_attach (GTK_TABLE (guiopt_look_table), cb_notabtext, 1, 2, 0, 1,
					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);

	cb_nodoubleclick = gtk_check_button_new_with_label (_("disable double-click in lists"));
	SET_TOOLTIP(cb_nodoubleclick, _("Make double clicks in lists have no effect. Useful for wheel-mouse users."));
	gtk_widget_show (cb_nodoubleclick);
	gtk_table_attach (GTK_TABLE (guiopt_look_table), cb_nodoubleclick, 1, 2, 1, 2,
					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);

/* deprecated:
	cb_showlogo = gtk_check_button_new_with_label (_("show donkey logo"));
	SET_TOOLTIP(cb_showlogo, _("Display the edonkey2000 logo at top. Set to true if you need extra space."));
	gtk_widget_show (cb_showlogo);
	gtk_table_attach (GTK_TABLE (guiopt_look_table), cb_showlogo, 0, 1, 0, 1,
					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					(GtkAttachOptions) (0), 5, 0);
*/
	cb_nogreen = gtk_check_button_new_with_label (_("I really hate the colour green"));
	SET_TOOLTIP(cb_nogreen, _("Set this to true to remove the default green color and use the system defaults and theme for the GUI (will become active on GUI restart)"));
	gtk_widget_show (cb_nogreen);
	gtk_table_attach (GTK_TABLE (guiopt_look_table), cb_nogreen, 0, 1, 1, 2,
					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					(GtkAttachOptions) (0), 5, 0);

	vbox10 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox10);
	gtk_box_pack_start (GTK_BOX (vbox9), vbox10, TRUE, TRUE, 0);

	hbox3 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox3);
	gtk_box_pack_start (GTK_BOX (vbox10), hbox3, TRUE, TRUE, 0);

	rotate_buttonbox = gtk_hbutton_box_new();
	gtk_widget_show(rotate_buttonbox);
	gtk_box_pack_start(GTK_BOX(hbox3), rotate_buttonbox, TRUE, TRUE, 0);
	new_icon_button_pack_and_signal_connect(&rotate_button,
	                                        rotate_buttonbox,
	                                        _("Rotate tab layout"),
	                                        options_rotate_tabs_button_clicked,
	                                        NULL,
	                                        1,
	                                        icon_refresh);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(rotate_buttonbox), GTK_BUTTONBOX_SPREAD);


/*  rotate_button = gtk_button_new_with_label (_("rotate"));
  gtk_widget_show (rotate_button);
  gtk_box_pack_start (GTK_BOX (hbox3), rotate_button, TRUE, TRUE, 5);
  GTK_WIDGET_SET_FLAGS (rotate_button, GTK_CAN_DEFAULT);
*/
	hbox5 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox5);
	gtk_box_pack_start (GTK_BOX (hbox3), hbox5, TRUE, TRUE, 0);

	guiopt_look_theme_frame = gtk_frame_new(_(" Icon theme "));
	gtk_widget_show(guiopt_look_theme_frame);
	gtk_container_set_border_width (GTK_CONTAINER(guiopt_look_theme_frame),5);
	gtk_box_pack_start (GTK_BOX (hbox3), guiopt_look_theme_frame, TRUE, TRUE, 5);

	omdirs = icons_get_installed_icon_themes ();
	if (omdirs)
	{
		GtkWidget	*hbuttonbox;
		GtkWidget	*menu_iconthemes; 
		GSList		*om_group = NULL;

		menu_iconthemes = gtk_menu_new();
		gtk_widget_show(menu_iconthemes);
		GTK_WIDGET_SET_FLAGS (menu_iconthemes, GTK_CAN_DEFAULT);

		om_iconthemes = gtk_option_menu_new();
		gtk_widget_show (om_iconthemes);
		GTK_WIDGET_SET_FLAGS (om_iconthemes, GTK_CAN_DEFAULT);
		gtk_container_set_border_width (GTK_CONTAINER(om_iconthemes),5);

		PROCESS_GSLIST (omdirs, gchar,
			if (item)
			{
				GtkWidget   *menu_item;
				gchar	    *mi_pathonly =g_dirname(item);
				gchar	    *mi_txt = g_strdup_printf (N_("%s (%s)"), g_basename(item), mi_pathonly);

				menu_item = gtk_radio_menu_item_new_with_label (om_group, mi_txt);
				GTK_WIDGET_SET_FLAGS (menu_item, GTK_CAN_DEFAULT);
				om_group  = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM(menu_item));
				gtk_widget_show(menu_item);
				gtk_signal_connect (GTK_OBJECT (menu_item), 	N_("activate"),
				GTK_SIGNAL_FUNC	(options_icontheme_selected),	(gpointer) item);
				gtk_menu_append (GTK_MENU(menu_iconthemes),	menu_item);
				g_free(mi_pathonly);
				g_free(mi_txt);
 			}
 		);
		/* note: do not free contents (used by "activate" signal callback)  */
		/* XXX - this leaks around 1-2kB, but this only happens once anyway */
		g_slist_free(omdirs);
		gtk_option_menu_set_menu (GTK_OPTION_MENU(om_iconthemes), menu_iconthemes);

		hbuttonbox = gtk_hbutton_box_new();
		gtk_widget_show(hbuttonbox);
		gtk_container_add(GTK_CONTAINER(hbuttonbox), om_iconthemes);

		gtk_container_add(GTK_CONTAINER(guiopt_look_theme_frame), hbuttonbox);
	} else g_printerr (N_("omdirs==NULL! (bug)\n"));

	return guiopt_look_frame;
}


/* options_page_make_guigeneral_frame
 *
 * creates the 'GUI General' frame and returns it
 *
 */

static GtkWidget *
options_page_make_guigeneral_frame (GtkTooltips *tooltips)
{
	GtkWidget *guiopt_general_frame;
	GtkWidget *guiopt_general_table;

	guiopt_general_frame = gtk_frame_new (_(" GUI General Options "));
	gtk_widget_show (guiopt_general_frame);
//	gtk_container_set_border_width (GTK_CONTAINER(guiopt_general_frame),5);

	guiopt_general_table = gtk_table_new (8, 2, FALSE);
	gtk_widget_show (guiopt_general_table);
	gtk_container_add (GTK_CONTAINER (guiopt_general_frame), guiopt_general_table);

	cb_verbose = gtk_check_button_new_with_label (_("verbose [core]"));
	SET_TOOLTIP(cb_verbose, _("toggle core verbose operation for debugging (note: option will not be saved by core)"));
	gtk_widget_show (cb_verbose);
	gtk_table_attach (GTK_TABLE (guiopt_general_table), cb_verbose, 0, 1, 0, 1,
					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					(GtkAttachOptions) (0), 5, 0);

	cb_verifyexit = gtk_check_button_new_with_label (_("ask before exit"));
	gtk_widget_show (cb_verifyexit);
	SET_TOOLTIP(cb_verifyexit, _("Ask for confirmation before closing the GUI"));
	gtk_table_attach (GTK_TABLE (guiopt_general_table), cb_verifyexit, 0, 1, 1, 2,
					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					(GtkAttachOptions) (0), 5, 0);

	cb_shutdowncoreonexit = gtk_check_button_new_with_label (_("shutdown core on exit"));
	gtk_widget_show (cb_shutdowncoreonexit);
	SET_TOOLTIP(cb_shutdowncoreonexit, _("Kill running core when the GUI is closed (otherwise it keeps running in the background"));

	gtk_table_attach (GTK_TABLE (guiopt_general_table), cb_shutdowncoreonexit, 0, 1, 2, 3,
					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					(GtkAttachOptions) (0), 5, 0);

	cb_spawncore = gtk_check_button_new_with_label (_("auto-spawn core"));
	gtk_widget_show (cb_spawncore);
	SET_TOOLTIP(cb_spawncore, _("Automatically start a donkey core at startup unless there is already a core running"));
	gtk_table_attach (GTK_TABLE (guiopt_general_table), cb_spawncore, 0, 1, 3, 4,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (0), 5, 0);

	cb_connectcore = gtk_check_button_new_with_label (_("auto-connect to core"));
	gtk_widget_show (cb_connectcore);
	SET_TOOLTIP(cb_connectcore, _("Automatically connect to running core at GUI startup"));
	gtk_table_attach (GTK_TABLE (guiopt_general_table), cb_connectcore, 0, 1, 4, 5,
					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					(GtkAttachOptions) (0), 5, 0);

	cb_restartcrashedcore = gtk_check_button_new_with_label (_("try to restart crashed core"));
	gtk_widget_show (cb_restartcrashedcore);
	SET_TOOLTIP(cb_restartcrashedcore, _("Try to spawn a new donkey core process in case of a core crash."));
	gtk_table_attach (GTK_TABLE (guiopt_general_table), cb_restartcrashedcore, 1, 2, 3, 4,
					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);

	cb_logtimestamp = gtk_check_button_new_with_label (_("show timestamps in status window"));
	SET_TOOLTIP(cb_logtimestamp, _("Display date and time in status window messages"));
	gtk_widget_show (cb_logtimestamp);
	gtk_table_attach (GTK_TABLE (guiopt_general_table), cb_logtimestamp, 1, 2, 1, 2,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);

	cb_nolinkstolog = gtk_check_button_new_with_label (_("do not write ed2k:// links to logfile"));
	gtk_widget_show (cb_nolinkstolog);
	SET_TOOLTIP(cb_nolinkstolog, _("Output ed2k links only to the status window, but not to the logfile"));
	gtk_table_attach (GTK_TABLE (guiopt_general_table), cb_nolinkstolog, 1, 2, 2, 3,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);

	cb_slowcon = gtk_check_button_new_with_label (_("minimise core-gui transfers"));
	SET_TOOLTIP(cb_slowcon, _("Set this TRUE if you are controlling the core from a slow network connection. It will reduce the transfers between the core and the gui to a minimum."));
	gtk_widget_show (cb_slowcon);
	gtk_table_attach (GTK_TABLE (guiopt_general_table), cb_slowcon, 1, 2, 4, 5,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);

	cb_linkdownload = gtk_check_button_new_with_label (_("download ed2k:// links immediately"));
	SET_TOOLTIP(cb_linkdownload, _("If set to true, links will be immediately downloaded. Otherwise, they will show up in the search page first."));
	gtk_widget_show (cb_linkdownload);
	gtk_table_attach (GTK_TABLE (guiopt_general_table), cb_linkdownload, 1, 2, 0, 1,
					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);

	return guiopt_general_frame;
}

/* options_page_make_page1_frame
 *
 *
 */

static GtkWidget *
options_page_make_page1_frame (GtkTooltips *tooltips)
{
	GtkWidget *guiopt_vbox;
	GtkWidget *vbox7;
	GtkWidget *scrollwin;

	guiopt_vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (guiopt_vbox);

	vbox7 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox7);
	gtk_box_pack_start (GTK_BOX (vbox7), guiopt_vbox, TRUE, TRUE, 5);
	gtk_container_set_border_width (GTK_CONTAINER(vbox7),5);

	/*********** get GUI General frame        **********/
	gtk_box_pack_start (GTK_BOX (guiopt_vbox), options_page_make_guigeneral_frame(tooltips), TRUE, TRUE, 0);

	/*********** get GUI Look and Feel frame  **********/
	gtk_box_pack_start (GTK_BOX (guiopt_vbox), options_page_make_lookfeel_frame(tooltips), TRUE, TRUE, 5);

	scrollwin = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scrollwin),
				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	// now add whole frame to scrolled window
	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrollwin), vbox7);
	gtk_widget_show (scrollwin);

	return scrollwin;
}


/* options_page_make_page2_frame
 *
 *
 */

static GtkWidget *
options_page_make_page2_frame (GtkTooltips *tooltips)
{
	GtkWidget *scrollwin;
	GtkWidget *coreopt_vbox;
	GtkWidget *coreopt_table;
	GtkWidget *coreopt_frame;
	GtkWidget *blanklabels[8];	/* 8 = number of ROWS */

	guint		row, rows = sizeof(blanklabels)/sizeof(GtkWidget*);

	coreopt_frame = gtk_frame_new(_(" Core options "));
	gtk_widget_show (coreopt_frame);
	gtk_container_set_border_width (GTK_CONTAINER(coreopt_frame),5);

	coreopt_vbox = gtk_vbox_new(TRUE, 0);
	gtk_widget_show (coreopt_vbox);
	gtk_container_add(GTK_CONTAINER(coreopt_frame), coreopt_vbox);
	coreopt_table = gtk_table_new (rows, 4, FALSE);
	gtk_box_pack_start(GTK_BOX(coreopt_vbox), coreopt_table, FALSE, TRUE, 5);
	gtk_widget_show (coreopt_table);

	/* make some empty labels in the last column so the entry widget field is not massively big */
	for (row=0; row<rows; row++)
	{
		blanklabels[row] = gtk_label_new (N_("     "));
		gtk_widget_show (blanklabels[row]);
		gtk_table_attach (GTK_TABLE (coreopt_table), blanklabels[row], 2, 3, row, row+1,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (0), 3, 0);
	}

	OPTIONS_ADD_ENTRY(coreopt_table, _(" nickname "), ew_nickname, 0, NULL);

	OPTIONS_ADD_SPINBUTTON(coreopt_table, _(" max d/l speed "), spinbutton_maxdlspeed,
		0.0, 0.0, 99999.0, 1.0, 1,
		_("The maximum speed with which you want the donkey to download, in kB/s. 0=no limit. "
			 "Note: The donkey has a ratio system. If you specify less than "
			 "10kB/s as max.upload speed, then your "
			 "download will be limited according to some fixed factor. "
			 "(Also: 1 kB/s = 8kbit/s. Line speeds are usually advertised in kbit/s, "
			 "so you need to divide by 8 to get your line speed in kB/s)")
	);

	OPTIONS_ADD_SPINBUTTON(coreopt_table, _(" max u/l speed"), spinbutton_maxulspeed,
		0.0, 0.0, 99999.0, 1.0, 2,
		_("The maximum speed with which you want the donkey to upload in kB/s. "
		 "You should set this to at least 10kB/s if you have a fast line, otherwise your "
		 "max. download speed will be limited. You should not set this value higher than "
		 "your upload line speed minus 3kB/s. "
		 "(Note: 1 kB/s = 8kbit/s. Line speeds are usually advertised in kbit/s, "
		 "so you need to divide by 8 to get your line speed in kB/s)")
	);

	OPTIONS_ADD_ENTRY(coreopt_table, _(" incoming dir"), ew_incoming, 3, NULL);

	OPTIONS_ADD_ENTRY(coreopt_table, _(" temp dir"), ew_temporary, 4, NULL);

	OPTIONS_ADD_SPINBUTTON(coreopt_table, _(" max connections "), spinbutton_maxcon,
		50.0, 50.0, 8192.0, 5.0, 5,
		_("The maximum number of sockets the donkey should use at any given time. "
			"Don't set this value too low! Try something between 350-500 and lower the "
			"value in case you get problems with your router (if you have one).")
	);

	OPTIONS_ADD_SPINBUTTON(coreopt_table, _(" port for other clients "), spinbutton_port,
		0.0, 1.0, 65535.0, 1.0, 6, _("The port the core listens on for other clients.")
	);

	OPTIONS_ADD_SPINBUTTON(coreopt_table, _(" port for controllers/gui "), spinbutton_adminport,
		0.0, 1.0, 65535.0, 1.0, 7, _("The port the core listens on for a GUI.")
	);

	scrollwin = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scrollwin),
				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	// now add whole frame to scrolled window
	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrollwin), coreopt_frame);
	gtk_widget_show (scrollwin);

  return scrollwin;
}


/* options_page_make_page3_frame
 *
 *
 */

static GtkWidget *
options_page_make_page3_frame (GtkTooltips *tooltips)
{
	GtkWidget *coreopt_vbox;
	GtkWidget *coreopt_table;
	GtkWidget *coreopt_frame;
	GtkWidget *blanklabels[11];	/* 10 = number of ROWS */
	GtkWidget *scrollwin;

	guint		row, rows = sizeof(blanklabels)/sizeof(GtkWidget*);

	coreopt_frame = gtk_frame_new(_(" More GUI options "));
	gtk_widget_show (coreopt_frame);
	gtk_container_set_border_width (GTK_CONTAINER(coreopt_frame),5);

	coreopt_vbox = gtk_vbox_new(TRUE, 0);
	gtk_widget_show (coreopt_vbox);
	gtk_container_add(GTK_CONTAINER(coreopt_frame), coreopt_vbox);
	coreopt_table = gtk_table_new (rows, 4, FALSE);
	gtk_box_pack_start(GTK_BOX(coreopt_vbox), coreopt_table, FALSE, TRUE, 5);
	gtk_widget_show (coreopt_table);

	/* make some empty labels in the last column so the entry widget field is not massively big */
	for (row=0; row<rows; row++)
	{
		blanklabels[row] = gtk_label_new (N_("     "));
		gtk_widget_show (blanklabels[row]);
		gtk_table_attach (GTK_TABLE (coreopt_table), blanklabels[row], 2, 3, row, row+1,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (0), 3, 0);
	}

	gtk_widget_show(coreopt_frame);

	OPTIONS_ADD_SPINBUTTON(coreopt_table, _(" default connect port "), spinbutton_defaultport,
		0.0, 1.0, 65535.0, 1.0, 0,
		_("This is the port that the GUI will connect to the core on if you "
			"have auto-connect at startup enabled. Otherwise this is the value "
			"that is the default value in the connect-to dialog.")
	);
	OPTIONS_ADD_ENTRY(coreopt_table, _(" default core username "), ew_defaultuser, 1, NULL);
	OPTIONS_ADD_ENTRY(coreopt_table, _(" default core password "), ew_defaultpass, 2, NULL);
	OPTIONS_ADD_ENTRY(coreopt_table, _(" default path to donkey binary "), ew_defaultdonkeypath, 3, NULL);

	OPTIONS_ADD_SPINBUTTON(coreopt_table, _(" server rotation interval "), spinbutton_serverrotate ,
		0.0, 0.0, 1440.0, 5.0, 4,
		_("Reconnect to another server after this many minutes (0=never). Minimum is 30 minutes.")
	);

	OPTIONS_ADD_ENTRY(coreopt_table, _(" url to favorite html serv-list "), ew_slistURL, 5, NULL);
	OPTIONS_ADD_ENTRY(coreopt_table, _(" url to secondary html serv-list "), ew_slistURL2, 6, NULL);
	OPTIONS_ADD_ENTRY(coreopt_table, _(" post-download command "), ew_completecommand, 7,
		_("Shell command that will be executed whenever a download is complete. "
		"The GUI will export the filename in the variable $ED2K_FN and the incoming directory "
		"in the variable $ED2K_IN")
	);

	OPTIONS_ADD_SPINBUTTON(coreopt_table, _(" clear status window interval"), spinbutton_clearswin,
		0.0, 0.0, 1440.0, 5.0, 8,
		_("Automatically clear the status log window after this many minutes (0=never).")
	);

	OPTIONS_ADD_ENTRY(coreopt_table, _(" default core host "), ew_defaulthost, 9, NULL);

	OPTIONS_ADD_SPINBUTTON(coreopt_table, _(" stop if free temp space is less than x MB"), spinbutton_minfreespace,
		0.0, 0.0, 9999.0, 0.0, 10,
		_("Pauses all downloads if there is less than x MegaBytes of free space on the temp folder partition (0=ignore).")
	);

	scrollwin = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scrollwin),
				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	// now add whole frame to scrolled window
	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrollwin), coreopt_frame);
	gtk_widget_show (scrollwin);

  return scrollwin;
}


/******************************************************************************
 *
 *   options_register_linkhandler_button_callback
 *
 */

static void
options_register_linkhandler_button_callback (GtkWidget *widget, gpointer data)
{
	const gchar *errmsg = NULL;
	const gchar *env = NULL;

	switch (GPOINTER_TO_UINT(data))
	{
		case OPTIONS_LINKHANDLER_INSTALL_KDE_HANDLER:
		{
			errmsg = linkhandler_install_kde();
			env = N_("KDE");
		}
		break;

		case OPTIONS_LINKHANDLER_INSTALL_GNOME1_HANDLER:
		{
			errmsg = linkhandler_install_gnome1();
			env = N_("Gnome 1");
		}
		break;

		case OPTIONS_LINKHANDLER_INSTALL_GNOME2_HANDLER:
		{
			errmsg = linkhandler_install_gnome2();
			env = N_("Gnome 2");
		}
		break;

		default: errmsg = N_("invalid case value in options_register_linkhandler_button_callback()!");
	}

	if (!errmsg)
	{
		gchar *msg = g_strdup_printf (_(" ed2k-link handler for %s installed successfully. "), env);
		ok_dialog(_(" install ed2k-link handler "), msg);
		G_FREE(msg);
	} else {
		gchar *msg = g_strdup_printf ( N_("\n%s%s:\n\n%s\n"),
		                               _(" An error occured while trying to install an ed2k-link handler for "),
		                               env, errmsg);
		ok_dialog(_(" install ed2k-link handler "), msg);
		G_FREE(msg);
	}
}



/******************************************************************************
 *
 *   options_page_make_linkhandler_options_frame
 *
 *   creates the frame with the linkhandler options
 *
 */

static GtkWidget *
options_page_make_linkhandler_options_frame (GtkTooltips *tooltips)
{
	GtkWidget *hbox;
	GtkWidget *label;
	
	hbox = gtk_hbox_new(FALSE, 5);
	gtk_widget_show (hbox);
	
	label = gtk_label_new (_(" If no GUI is running...   "));
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 5);

	radio1_lh = gtk_radio_button_new_with_label(NULL, _(" ask what to do "));
	gtk_widget_show(radio1_lh);
	gtk_box_pack_start(GTK_BOX(hbox), radio1_lh, TRUE, TRUE, 5);
	
	radio2_lh = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON(radio1_lh), _(" start GUI "));
	gtk_widget_show(radio2_lh);	
	gtk_box_pack_start(GTK_BOX(hbox), radio2_lh, TRUE, TRUE, 5);
	
	radio3_lh = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON(radio1_lh), _(" silently remember link "));
	gtk_widget_show(radio3_lh);
	gtk_box_pack_start(GTK_BOX(hbox), radio3_lh, TRUE, TRUE, 5);
	
	return hbox;
}

/******************************************************************************
 *
 *   options_page_make_linkhandler_page_frame
 *
 *   creates the link handler setup frame for the options page
 *
 */

static GtkWidget *
options_page_make_linkhandler_page_frame (GtkTooltips *tooltips)
{
	GtkWidget *scrollwin;
	GtkWidget *vbox;
	GtkWidget *vbuttonbox;
	GtkWidget *frame;
	GtkWidget *kde_button;
	GtkWidget *gnome1_button;
	GtkWidget *gnome2_button;
	GtkWidget *label;
	GtkWidget *emptylabel;

	GtkWidget *btnframe;
	GtkWidget *hbox1, *hbox2, *hbox3;
	GtkWidget *kde_hbox, *kde_bbox, *gnome1_hbox, *gnome1_bbox, *gnome2_hbox, *gnome2_bbox;

	frame = gtk_frame_new(_(" ed2k-link handler "));
	gtk_widget_show (frame);
	gtk_container_set_border_width (GTK_CONTAINER(frame),15);

	scrollwin = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrollwin);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scrollwin),
				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrollwin), frame);

	vbox = gtk_vbox_new(FALSE,15);
	gtk_widget_show(vbox);
	gtk_container_add(GTK_CONTAINER(frame), vbox);

	gtk_box_pack_start(GTK_BOX(vbox), options_page_make_linkhandler_options_frame(tooltips), FALSE, FALSE, 5);
	
	btnframe = gtk_frame_new(_(" Register link handler "));
	gtk_box_pack_end(GTK_BOX(vbox), btnframe, FALSE, FALSE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(btnframe), 15);
	gtk_widget_show(btnframe); 

	vbuttonbox = gtk_vbox_new(FALSE, 15);
	gtk_widget_show (vbuttonbox);
	
	gtk_container_add(GTK_CONTAINER(btnframe), vbuttonbox);

	label = gtk_label_new(_(" You can register the GUI as handler for ed2k-links \n"
	                        " with your desktop environment (KDE,GNOME). Then you \n"
	                        " can just click on ed2k-links in your browser, and \n"
	                        " the link will be passed to the GUI. This only works \n"
	                        " with browsers who honour your desktop environments' \n"
	                        " settings, ie. Galeon and Konqueror will work,\n"
	                        " but Mozilla and Opera will probably not work."));
	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(vbuttonbox), label, TRUE, TRUE, 10);	
	
	hbox1 = gtk_hbox_new(FALSE, 15);
	gtk_box_pack_start(GTK_BOX(vbuttonbox), hbox1, TRUE, TRUE, 10);
	
	kde_hbox =  make_icon_and_label_hbox(_(" Install ed2k:// link handler for KDE "), icon_kde);
	gtk_widget_show(kde_hbox);

	gtk_box_pack_start(GTK_BOX(hbox1), kde_hbox, TRUE, TRUE, 50);
	kde_bbox = gtk_hbutton_box_new();
	emptylabel = gtk_label_new(N_(" "));
	gtk_widget_show(emptylabel);
	gtk_box_pack_start(GTK_BOX(hbox1), kde_bbox, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(hbox1), emptylabel, FALSE, FALSE, 10);

	gtk_widget_show(kde_bbox);

	new_button_pack_and_signal_connect(&kde_button, kde_bbox, _(" Install "), 
			options_register_linkhandler_button_callback,
			GUINT_TO_POINTER(OPTIONS_LINKHANDLER_INSTALL_KDE_HANDLER), 1);

	hbox2 = gtk_hbox_new(FALSE, 15);
	gtk_box_pack_start(GTK_BOX(vbuttonbox), hbox2, TRUE, TRUE, 10);

	gnome1_hbox =  make_icon_and_label_hbox(_(" Install ed2k:// link handler for GNOME 1 "), icon_gnome1);
	gtk_widget_show(gnome1_hbox);

	gtk_box_pack_start(GTK_BOX(hbox2), gnome1_hbox, TRUE, TRUE, 50);
	gnome1_bbox = gtk_hbutton_box_new();
	emptylabel = gtk_label_new(N_(" "));
	gtk_widget_show(emptylabel);
	gtk_box_pack_start(GTK_BOX(hbox2), gnome1_bbox, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(hbox2), emptylabel, FALSE, FALSE, 10);
	gtk_widget_show(gnome1_bbox);


	new_button_pack_and_signal_connect (
		&gnome1_button,
		gnome1_bbox,
		_(" Install "),
		options_register_linkhandler_button_callback,
		GUINT_TO_POINTER(OPTIONS_LINKHANDLER_INSTALL_GNOME1_HANDLER),
		1
	);

	hbox3 = gtk_hbox_new(FALSE, 15);
	gtk_box_pack_start(GTK_BOX(vbuttonbox), hbox3, TRUE, TRUE, 10);

	gnome2_hbox =  make_icon_and_label_hbox(_(" Install ed2k:// link handler for GNOME 2 "), icon_gnome2);
	gtk_widget_show(gnome2_hbox);

	gtk_box_pack_start(GTK_BOX(hbox3), gnome2_hbox, TRUE, TRUE, 50);
	gnome2_bbox = gtk_hbutton_box_new();
	gtk_widget_show(gnome2_bbox);
        
	emptylabel = gtk_label_new(N_(" "));
	gtk_widget_show(emptylabel);
	gtk_box_pack_start(GTK_BOX(hbox3), gnome2_bbox, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(hbox3), emptylabel, FALSE, FALSE, 10);


	new_button_pack_and_signal_connect (
		&gnome2_button,
		gnome2_bbox,
		_(" Install "),
		options_register_linkhandler_button_callback,
		GUINT_TO_POINTER(OPTIONS_LINKHANDLER_INSTALL_GNOME2_HANDLER),
		1
	);

	gtk_widget_show(hbox1);
	gtk_widget_show(hbox2);
	gtk_widget_show(hbox3);
	return scrollwin;
}



/* options_page_create
 *
 * (new version by Axel C)
 *
 *
 */

GtkWidget *
options_page_create (void)
{
	GtkWidget *vbox5;
	GtkWidget *opt_buttonbox;
	GtkWidget *ok_button;
	GtkWidget *cancel_button;
	GtkWidget *refresh_button;
	GtkWidget *vbox6;
	GtkWidget *notebook1;
	GtkWidget *label61;
	GtkWidget *label62;
	GtkWidget *label999;
	GtkWidget *label_linkhandler;
	GtkTooltips *tooltips;

	tooltips = gtk_tooltips_new();

	vbox5 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox5);

	opt_buttonbox = gtk_hbutton_box_new ();
	gtk_widget_show (opt_buttonbox);
	gtk_box_pack_end (GTK_BOX (vbox5), opt_buttonbox, FALSE, TRUE, 10);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (opt_buttonbox), GTK_BUTTONBOX_END);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (opt_buttonbox), 5);


	new_icon_button_pack_and_signal_connect(&ok_button, opt_buttonbox, _(" OK "), options_ok_button_clicked, NULL, 1,
					icon_ok);

	new_icon_button_pack_and_signal_connect(&cancel_button, opt_buttonbox, _(" Cancel "), options_cancel_button_clicked, NULL, 1,
					icon_cancel);
	new_icon_button_pack_and_signal_connect(&refresh_button, opt_buttonbox, _(" Refresh "), options_refresh_button_clicked, NULL, 1,
					icon_refresh);

	vbox6 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox6);
	gtk_box_pack_start (GTK_BOX (vbox5), vbox6, TRUE, TRUE, 0);

	notebook1 = gtk_notebook_new ();
	gtk_widget_show (notebook1);
	gtk_container_add (GTK_CONTAINER (vbox6), notebook1);
	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook1), GTK_POS_LEFT);

	/********** get page 1 frame (GUI I) *******************/
	gtk_container_add (GTK_CONTAINER (notebook1), options_page_make_page1_frame(tooltips));

	/********** get page 3 frame (GUI II) ******************/
	gtk_container_add (GTK_CONTAINER (notebook1), options_page_make_page3_frame(tooltips));

	/********** get next frame (linkhandler)  *******************/
	gtk_container_add (GTK_CONTAINER (notebook1), options_page_make_linkhandler_page_frame(tooltips));

	/********** get page 2 frame (CORE)  *******************/
	gtk_container_add (GTK_CONTAINER (notebook1), options_page_make_page2_frame(tooltips));

	label61 = make_icon_and_label_hbox(_(" GUI 1 "),icon_options);
	gtk_widget_show (label61);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), label61);
//	gtk_label_set_justify (GTK_LABEL (label61), GTK_JUSTIFY_LEFT);

	label999 = make_icon_and_label_hbox(_(" GUI 2 "),icon_options);
	gtk_widget_show (label999);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), label999);
//	gtk_label_set_justify (GTK_LABEL (label999), GTK_JUSTIFY_LEFT);

	label_linkhandler = make_icon_and_label_hbox(_(" GUI 3 "),icon_options);
	gtk_widget_show(label_linkhandler);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 2), label_linkhandler);

	label62 = make_icon_and_label_hbox(_(" Core "),icon_options);
	gtk_widget_show (label62);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 3), label62);
//	gtk_label_set_justify (GTK_LABEL (label62), GTK_JUSTIFY_LEFT);

	return vbox5;
}

